/*
 * Copyright (c) 2010 JWT -- All rights reserved
 */
package ar.com.zauber.taglibs.dateutlils;

import java.util.HashMap;
import java.util.Map;

import org.apache.commons.lang.Validate;

/**
 * Builds the string representation of the date of a given locate
 * 
 * @author Pablo Martin Grigolatto
 * @since Jul 23, 2010
 */
public class RelativeDateFormatterStringBuilder {

    private static final Map<TimeUnit, String> TRANSLATIONS_ES;
    private static final Map<TimeUnit, String> TRANSLATIONS_EN;
    private static final Map<TimeUnit, String> TRANSLATIONS_PT;
    
    private TimeUnit unit = null;
    private Locate locate = null;
    private Boolean past = null;
    private Long amount = null;
    private boolean plural = false;

    //TODO esto podra inyectarse desde otro lado
    static {
        TRANSLATIONS_ES =  new HashMap<TimeUnit, String>();
        TRANSLATIONS_ES.put(TimeUnit.DAY, "da");
        TRANSLATIONS_ES.put(TimeUnit.HOUR, "hora");
        TRANSLATIONS_ES.put(TimeUnit.MINUTE, "minuto");
        TRANSLATIONS_ES.put(TimeUnit.SECOND, "segundo");
        TRANSLATIONS_ES.put(TimeUnit.WEEK, "semana");
        TRANSLATIONS_ES.put(TimeUnit.YEAR, "ao");
        
        TRANSLATIONS_EN =  new HashMap<TimeUnit, String>();
        TRANSLATIONS_EN.put(TimeUnit.DAY, "day");
        TRANSLATIONS_EN.put(TimeUnit.HOUR, "hour");
        TRANSLATIONS_EN.put(TimeUnit.MINUTE, "minute");
        TRANSLATIONS_EN.put(TimeUnit.SECOND, "second");
        TRANSLATIONS_EN.put(TimeUnit.WEEK, "week");
        TRANSLATIONS_EN.put(TimeUnit.YEAR, "year");
        
        TRANSLATIONS_PT =  new HashMap<TimeUnit, String>();
        TRANSLATIONS_PT.put(TimeUnit.DAY, "dia");
        TRANSLATIONS_PT.put(TimeUnit.HOUR, "hora");
        TRANSLATIONS_PT.put(TimeUnit.MINUTE, "minuto");
        TRANSLATIONS_PT.put(TimeUnit.SECOND, "segundo");
        TRANSLATIONS_PT.put(TimeUnit.WEEK, "semana");
        TRANSLATIONS_PT.put(TimeUnit.YEAR, "ano");
    }
    
    /** Creates the RelativeDateFormatterStringBuilder. */
    public RelativeDateFormatterStringBuilder(final String locate) {
        setLocate(locate);
    }
    
    /** validates the locate string */
    private void setLocate(final String string) {
        for (Locate l : Locate.values()) {
            if (l.getName().equalsIgnoreCase(string)) {
                this.locate = l;
                return;
            }
        }
        throw new IllegalArgumentException("Unsupported language: " + locate);
    }

    /** the date is in the past */
    public final void setPast() {
         this.past = Boolean.TRUE;
    }

    /** the date is in the future */
    public final void setFuture() {
        this.past = Boolean.FALSE;
    }

    /** the amount of units */
    public final void setAmount(final long amount) {
        this.amount = Long.valueOf(amount);
        this.plural = Long.valueOf(1) != this.amount;
    }

    public final void setTimeUnit(final TimeUnit timeUnit) {
        this.unit = timeUnit;
    }
    
     /** @return the formatted string */
    public final String build() {
        validateState();
        final StringBuilder sb = new StringBuilder();
        //TODO podra ser una collection de strategies 
        //     con los distintos algoritmos de orden
        switch (this.locate) {
            case EN: 
                if(!past) {
                    sb.append("in ");
                }
                addAmountAndUnit(sb, TRANSLATIONS_EN);
                if(past) {
                    sb.append(" ago");
                }
                break;
            case ES:
                if(past) {
                    sb.append("hace ");
                } else {
                    sb.append("en "); 
                }
                addAmountAndUnit(sb, TRANSLATIONS_ES);
                break;
            case PT:
                if(past) {
                    sb.append("faz ");
                } else {
                    sb.append("em "); 
                }
                addAmountAndUnit(sb, TRANSLATIONS_PT);
                break;
            default:
                throw new IllegalArgumentException(
                        "Unsupported language: " + this.locate);
        }
        return sb.toString();
    }

    /** adds the amount and unit translations */
    private void addAmountAndUnit(
            final StringBuilder sb, final Map<TimeUnit, String> map) {
        sb.append(amount);
        sb.append(" ");
        sb.append(map.get(unit));
        if(plural) {
            sb.append("s");
        }
    }

    /** validates internal state */
    private void validateState() {
        Validate.notNull(locate, "the Locate can not be null");
        Validate.notNull(unit, "the Time Unit can not be null");
        Validate.notNull(past, "the Past can not be null"); // ;)
        Validate.notNull(amount, "the Amount can not be null");
    }
    
}

/**
 * Supported locates
 * 
 * @author Pablo Martin Grigolatto
 * @since Jul 23, 2010
 */
enum Locate {
    /** locates */
    EN("en"), ES("es"), PT("pt");

    private final String name;
    
    /** Creates the Locate. */
    private Locate(final String name) {
        this.name = name;
    }
    
    public String getName() {
        return name;
    }
}
