/*
 * Copyright (c) 2005 Zauber  -- All rights reserved
 */
package ar.com.zauber.taglibs.dateutlils;

import java.io.IOException;
import java.util.Date;

import javax.servlet.jsp.JspException;
import javax.servlet.jsp.tagext.TagSupport;

import org.apache.commons.lang.StringEscapeUtils;
import org.apache.commons.lang.StringUtils;
import org.apache.commons.lang.Validate;
import org.apache.taglibs.standard.lang.support.ExpressionEvaluatorManager;

/**
 * Relative date tag
 *
 * @author Juan F. Codagnone
 * @since Nov 13, 2005
 */
public class RelativeDateTag extends TagSupport {
    private String locate;
    private Date date;

    /** @see TagSupport#doStartTag() */
    @Override
    public final int doStartTag() throws JspException {
        Validate.notNull(date, "date wasn't set");
        Validate.isTrue(StringUtils.isNotEmpty(locate), "locate wasn't set");
        
        try {
            pageContext.getOut().write(
                StringEscapeUtils.escapeHtml(
                    new RelativeDateFormatter().format(locate, date)));
        } catch(final IOException e) {
            throw new JspException(e);
        }
        
        return SKIP_BODY;
    }

    /** @throws JspException */
    public final void setDate(final String date) throws JspException {
        final String s = (String) ExpressionEvaluatorManager.evaluate(
                "date", date, String.class, pageContext);
        this.date = new Date(Long.parseLong(s));
    }
    
    public final void setLocate(final String locate) {
        this.locate = locate;
    }

    public final String getDate() {
        return date.toString();
    }
    
}
